<?php

class ForumWatch_ControllerAdmin_ForumWatchGroup extends XenForo_ControllerAdmin_Abstract
{
	public function actionIndex()
	{
		$forumWatchGroupModel = $this->_getForumWatchGroupModel();
		$forumWatchesGroup = $forumWatchGroupModel->getForumWatchesGroup(array(

		), array(
				'join' => ForumWatch_Model_ForumWatchGroup::FETCH_USER_GROUP | ForumWatch_Model_ForumWatchGroup::FETCH_FORUM,
				'order' => 'natural',
		));

		$viewParams = array(
				'forumWatchesGroup' => $forumWatchesGroup
		);

		return $this->responseView('ForumWatch_ViewAdmin_ForumWatchGroup_List', 'forumwatch_xf_forum_watch_group_list', $viewParams);
	}

	public function actionAdd()
	{
		return $this->_actionAddOrEdit(array(
				'notification_method' => 0,
		));
	}

	public function actionEdit()
	{
		$userGroupId = $this->_input->filterSingle('user_group_id', XenForo_Input::UINT);
		$forumId = $this->_input->filterSingle('forum_id', XenForo_Input::UINT);
		$forumWatchGroup = $this->_getForumWatchGroupOrError($userGroupId, $forumId);

		return $this->_actionAddOrEdit($forumWatchGroup);
	}

	protected function _actionAddOrEdit($forumWatchGroup)
	{
		$userGroups = $this->getModelFromCache('XenForo_Model_UserGroup')->getAllUserGroupTitles();

		$nodes = $this->getModelFromCache('XenForo_Model_Node')->getAllNodes();
		$forums = array();
		foreach ($nodes as $node)
		{
			if ($node['node_type_id'] == 'Forum')
			{
				$forums[] = array(
						'value' => $node['node_id'],
						'label' => $node['title'],
						'depth' => $node['depth'],
				);
			}
			else
			{
				$forums[] = array(
						'value' => $node['node_id'],
						'label' => $node['title'],
						'depth' => $node['depth'],
						'disabled' => true,
				);
			}
		}

		$viewParams = array(
				'forumWatchGroup' => $forumWatchGroup,
				'userGroups' => $userGroups,
				'forums' => $forums,
		);

		return $this->responseView('ForumWatch_ViewAdmin_ForumWatchGroup_Edit', 'forumwatch_xf_forum_watch_group_edit', $viewParams);
	}

	public function actionSave()
	{
		$this->_assertPostOnly();

		$userGroupId = $this->_input->filterSingle('existing_user_group_id', XenForo_Input::UINT);
		$forumId = $this->_input->filterSingle('existing_forum_id', XenForo_Input::UINT);

		try
		{
			$forumWatchGroup = $this->_getForumWatchGroupOrError($userGroupId, $forumId);
		}
		catch (XenForo_ControllerResponse_Exception $e)
		{
			// ignore
		}

		$dw = $this->_getForumWatchGroupDataWriter();

		if (!empty($forumWatchGroup))
		{
			$dw->setExistingData($forumWatchGroup, true);
		}

		// get regular fields from input data
		$dwInput = $this->_input->filter(array(
				'user_group_id' => XenForo_Input::UINT,
				'forum_id' => XenForo_Input::UINT,
				'reply_subscribe' => XenForo_Input::UINT,
				'notification_method' => XenForo_Input::UINT
		));
		$dw->bulkSet($dwInput);

		$dw->save();

		return $this->responseRedirect(
				XenForo_ControllerResponse_Redirect::SUCCESS,
				XenForo_Link::buildAdminLink('group-forum-watches')
		);
	}

	public function actionDelete()
	{
		$userGroupId = $this->_input->filterSingle('user_group_id', XenForo_Input::UINT);
		$forumId = $this->_input->filterSingle('forum_id', XenForo_Input::UINT);
		$forumWatchGroup = $this->_getForumWatchGroupOrError($userGroupId, $forumId, array(
				'join' => ForumWatch_Model_ForumWatchGroup::FETCH_USER_GROUP | ForumWatch_Model_ForumWatchGroup::FETCH_FORUM,
		));

		if ($this->isConfirmedPost())
		{
			$dw = $this->_getForumWatchGroupDataWriter();
			$dw->setExistingData($forumWatchGroup, true);
			$dw->delete();

			return $this->responseRedirect(
					XenForo_ControllerResponse_Redirect::SUCCESS,
					XenForo_Link::buildAdminLink('group-forum-watches')
			);
		}
		else
		{
			$viewParams = array(
					'forumWatchGroup' => $forumWatchGroup
			);

			return $this->responseView('ForumWatch_ViewAdmin_ForumWatchGroup_Delete', 'forumwatch_xf_forum_watch_group_delete', $viewParams);
		}
	}

	protected function _getForumWatchGroupOrError($userGroupId, $forumId, array $fetchOptions = array())
	{
		$forumWatchGroup = $this->_getForumWatchGroupModel()->getForumWatchGroupByIds($userGroupId, $forumId, $fetchOptions);

		if (empty($forumWatchGroup))
		{
			throw $this->responseException($this->responseError(new XenForo_Phrase('forumwatch_xf_forum_watch_group_not_found'), 404));
		}

		return $forumWatchGroup;
	}

	/**
	 * @return ForumWatch_Model_ForumWatchGroup
	 */
	protected function _getForumWatchGroupModel()
	{
		return $this->getModelFromCache('ForumWatch_Model_ForumWatchGroup');
	}

	/**
	 * @return ForumWatch_DataWriter_ForumWatchGroup
	 */
	protected function  _getForumWatchGroupDataWriter()
	{
		return XenForo_DataWriter::create('ForumWatch_DataWriter_ForumWatchGroup');
	}

}