<?php

class ForumWatch_DataWriter_ForumWatchGroup extends XenForo_DataWriter
{
	protected function _getFields()
	{
		return array(
				'xf_forum_watch_group' => array(
						'user_group_id' => array(
								'type' => XenForo_DataWriter::TYPE_UINT,
								'required' => true,
								'verification' => array('$this', '_verifyUserGroupId'),
						),
						'forum_id' => array(
								'type' => XenForo_DataWriter::TYPE_UINT,
								'required' => true,
								'verification' => array('$this', '_verifyForumId'),
						),
						'reply_subscribe' => array('type' => XenForo_DataWriter::TYPE_UINT, 'required' => true),
						'notification_method' => array('type' => XenForo_DataWriter::TYPE_UINT, 'required' => true, 'default' => 0),
				)
		);
	}

	protected function _getExistingData($data)
	{
		throw new XenForo_Exception('ForumWatch_DataWriter_ForumWatchGroup::_getExistingData is not implemented');
	}

	protected function _getUpdateCondition($tableName)
	{
		$conditions = array();

		foreach (array('user_group_id', 'forum_id') as $field)
		{
			$conditions[] = $field . ' = ' . $this->_db->quote($this->getExisting($field));
		}

		return implode(' AND ', $conditions);
	}

	protected function _verifyUserGroupId(&$userGroupId)
	{
		if (empty($userGroupId))
		{
			$this->error(new XenForo_Phrase('requested_user_group_not_found'), 'user_group_id');
			return false;
		}

		$userGroup = $this->getModelFromCache('XenForo_Model_UserGroup')->getUserGroupById($userGroupId);
		if (empty($userGroup))
		{
			$this->error(new XenForo_Phrase('requested_user_group_not_found'), 'user_group_id');
			return false;
		}

		return true;
	}

	protected function _verifyForumId(&$forumId)
	{
		if (empty($forumId))
		{
			$this->error(new XenForo_Phrase('requested_forum_not_found'), 'user_group_id');
			return false;
		}

		$forum = $this->getModelFromCache('XenForo_Model_Forum')->getForumById($forumId);
		if (empty($forum))
		{
			$this->error(new XenForo_Phrase('requested_forum_not_found'), 'user_group_id');
			return false;
		}

		return true;
	}

	protected function _preSave()
	{
		if ($this->isInsert())
		{
			$existing = $this->_getForumWatchGroupModel()->getForumWatchGroupByIds($this->get('user_group_id'), $this->get('forum_id'));
			if (!empty($existing))
			{
				$this->error(new XenForo_Phrase('forumwatch_existing_group_watch_found'), 'forum_id');
			}
		}

		return parent::_preSave();
	}
	
	protected function _postSave()
	{
		$this->_getForumWatchGroupModel()->rebuildCacheForumGroup();
		
		return parent::_postSave();
	}
	
	protected function _postDelete()
	{
		$this->_getForumWatchGroupModel()->rebuildCacheForumGroup();
	
		return parent::_postDelete();
	}

	/**
	 * @return ForumWatch_Model_ForumWatchGroup
	 */
	protected function _getForumWatchGroupModel()
	{
		return $this->getModelFromCache('ForumWatch_Model_ForumWatchGroup');
	}

}