<?php

class ForumWatch_Extend_ControllerPublic_Forum extends XFCP_ForumWatch_Extend_ControllerPublic_Forum
{
	public function actionIndex()
	{
		$response = parent::actionIndex();

		if ($response instanceof XenForo_ControllerResponse_View AND !empty($response->params['forum']))
		{
			$response->params['canWatchForum'] = $this->_getForumModel()->canWatchForum($response->params['forum']);
		}
			
		return $response;
	}

	public function actionWatchConfirm()
	{
		$ftpHelper = $this->getHelper('ForumThreadPost');
		$forumId = $this->_input->filterSingle('node_id', XenForo_Input::UINT);
		$forumName = $this->_input->filterSingle('node_name', XenForo_Input::STRING);
		$forum = $ftpHelper->assertForumValidAndViewable($forumId ? $forumId : $forumName);

		if (!$this->_getForumModel()->canViewForum($forum))
		{
			return $this->responseNoPermission();
		}
		
		if (!$this->_getForumModel()->canWatchForum($forum))
		{
			return $this->responseNoPermission();
		}

		$forumWatch = $this->_getForumWatchModel()->getUserForumWatchByForumId(
				XenForo_Visitor::getUserId(), $forum['node_id']
		);

		$viewParams = array(
				'forum' => $forum,
				'forumWatch' => $forumWatch,
				'nodeBreadCrumbs' => $ftpHelper->getNodeBreadCrumbs($forum),
		);

		return $this->responseView('', 'forumwatch_forum_watch', $viewParams);
	}

	public function actionWatch()
	{
		$this->_assertPostOnly();

		$ftpHelper = $this->getHelper('ForumThreadPost');
		$forumId = $this->_input->filterSingle('node_id', XenForo_Input::UINT);
		$forumName = $this->_input->filterSingle('node_name', XenForo_Input::STRING);
		$forum = $ftpHelper->assertForumValidAndViewable($forumId ? $forumId : $forumName);

		if (!$this->_getForumModel()->canViewForum($forum))
		{
			return $this->responseNoPermission();
		}
		
		if (!$this->_getForumModel()->canWatchForum($forum))
		{
			return $this->responseNoPermission();
		}

		$input = $this->_input->filter(array(
				'stop' => XenForo_Input::STRING,
				'reply_subscribe' => XenForo_Input::UINT,
				'notification_method' => XenForo_Input::UINT,
		));
		$userId = XenForo_Visitor::getUserId();
		$forumWatch = $this->_getForumWatchModel()->getUserForumWatchByForumId($userId, $forum['node_id']);

		$dw = XenForo_DataWriter::create('ForumWatch_DataWriter_ForumWatch');
		if ($forumWatch) $dw->setExistingData($forumWatch, true);

		if ($input['stop'] == 'stop')
		{
			$dw->delete();
		}
		else
		{
			$dw->set('user_id', $userId);
			$dw->set('forum_id', $forum['node_id']);
			$dw->set('reply_subscribe', $input['reply_subscribe']);
			$dw->set('notification_method', $input['notification_method']);
			$dw->save();
		}

		return $this->responseRedirect(
				XenForo_ControllerResponse_Redirect::SUCCESS,
				XenForo_Link::buildPublicLink('forums', $forum),
				null,
				array('linkPhrase' => (empty($input['stop']) ? new XenForo_Phrase('forumwatch_unwatch_forum') : new XenForo_Phrase('forumwatch_watch_forum')))
		);
	}

	/**
	 * @return ForumWatch_Model_ForumWatch
	 */
	protected function _getForumWatchModel()
	{
		return $this->getModelFromCache('ForumWatch_Model_ForumWatch');
	}

	protected function _getForumFetchOptions()
	{
		$fetchOptions = parent::_getForumFetchOptions();
		$fetchOptions['ForumWatch_watchUserId'] = XenForo_Visitor::getUserId();

		return $fetchOptions;
	}
}