<?php

class ForumWatch_Extend_ControllerPublic_Watched extends XFCP_ForumWatch_Extend_ControllerPublic_Watched
{
	public function actionForums()
	{
		return $this->responseRedirect(
				XenForo_ControllerResponse_Redirect::RESOURCE_CANONICAL_PERMANENT
				, XenForo_Link::buildPublicLink('full:watched/forums/all')
		);
	}

	public function actionForumsAll()
	{
		$visitor = XenForo_Visitor::getInstance();
		$nodeModel = $this->getModelFromCache('XenForo_Model_Node');

		$nodeList = $nodeModel->getNodeDataForListDisplay(false, 0);
		$nodeHandlerForum = new ForumWatch_NodeHandler_Forum();
		$nodeHandlerForum->ForumWatch_setShowAllInWatchedList(ForumWatch_Option::get('showAllInWatchedList'));

		$nodeHandlerOther = new ForumWatch_NodeHandler_Other();
		foreach (array_keys($nodeList['nodeHandlers']) as $nodeType)
		{
			switch ($nodeType)
			{
				case 'Category':
					break;
				case 'Forum':
					$nodeList['nodeHandlers'][$nodeType] = $nodeHandlerForum;
					break;
				default:
					$nodeList['nodeHandlers'][$nodeType] = $nodeHandlerOther;
			}
		}

		$watchedForums = $this->_getForumWatchModel()->getForumsWatchedByUser($visitor->get('user_id'));
		foreach ($nodeList['nodesGrouped'] as &$nodes)
		{
			foreach ($nodes as &$node)
			{
				if ($node['node_type_id'] == 'Forum')
				{
					if (isset($watchedForums[$node['node_id']]))
					{
						$node['forum_watch'] = $watchedForums[$node['node_id']];
					}

					$node['forum_watch_group'] = $this->_getForumWatchGroupModel()->getUserInWatchingGroupsOfForum(
							$visitor->get('user_id'),
							$node['node_id']);
				}
			}
		}

		$viewParams = array(
				'nodeList' => $nodeList,
		);

		return $this->responseView('ForumWatch_ViewPublic_Watched_ForumsAll', 'forumwatch_list', $viewParams);
	}

	public function actionForumsUpdate()
	{
		$this->_assertPostOnly();

		$input = $this->_input->filter(array(
				'forum_ids' => array(XenForo_Input::UINT, 'array' => true),
				'do' => XenForo_Input::STRING
		));

		/* @var $forumModel XenForo_Model_Forum */
		$forumModel = $this->getModelFromCache('XenForo_Model_Forum');

		$userId = XenForo_Visitor::getUserId();
		$watch = $this->_getForumWatchModel()->getUserForumWatchByForumIds($userId, $input['forum_ids']);
		$forums = $forumModel->getForumsByIds($input['forum_ids']);

		foreach ($watch AS $forumWatch)
		{
			if (empty($forums[$forumWatch['forum_id']]))
			{
				continue;
			}
			$forumRef = $forums[$forumWatch['forum_id']];


			if (!$forumModel->canWatchForum($forumRef))
			{
				continue;
			}

			/* @var $dw ForumWatch_DataWriter_ForumWatch */
			$dw = XenForo_DataWriter::create('ForumWatch_DataWriter_ForumWatch');
			$dw->setExistingData($forumWatch, true);

			switch ($input['do'])
			{
				case 'no_reply':
					$dw->set('reply_subscribe', 0);
					$dw->save();
					break;
				case 'reply':
					$dw->set('reply_subscribe', 1);
					$dw->save();
					break;
				case 'email_only':
					$dw->set('notification_method', 0);
					$dw->save();
					break;
				case 'alert_only':
					$dw->set('notification_method', 1);
					$dw->save();
					break;
				case 'email_and_alert':
					$dw->set('notification_method', 2);
					$dw->save();
					break;
				case 'stop':
					$dw->delete();
					break;
			}
		}

		return $this->responseRedirect(
				XenForo_ControllerResponse_Redirect::SUCCESS,
				$this->getDynamicRedirect(XenForo_Link::buildPublicLink('watched/forums/all'))
		);
	}

	/**
	 * @return ForumWatch_Model_ForumWatch
	 */
	private function _getForumWatchModel()
	{
		return $this->getModelFromCache('ForumWatch_Model_ForumWatch');
	}

	/**
	 * @return ForumWatch_Model_ForumWatchGroup
	 */
	private function _getForumWatchGroupModel()
	{
		return $this->getModelFromCache('ForumWatch_Model_ForumWatchGroup');
	}
}