<?php

class ForumWatch_Model_ForumWatch extends XenForo_Model
{
	public function getUserForumWatchByForumId($userId, $forumId)
	{
		return $this->_getDb()->fetchRow('
				SELECT *
				FROM xf_forum_watch
				WHERE user_id = ?
				AND forum_id = ?
				', array($userId, $forumId));
	}

	public function getUserForumWatchByForumIds($userId, array $forumIds)
	{
		if (empty($forumIds)) return array();

		return $this->fetchAllKeyed('
				SELECT *
				FROM xf_forum_watch
				WHERE user_id = ?
				AND forum_id IN (' . $this->_getDb()->quote($forumIds) . ')
				', 'forum_id', $userId);
	}

	public function getForumsWatchedByUser($userId)
	{
		return $this->fetchAllKeyed('
				SELECT forum_watch.*
				FROM xf_forum_watch AS forum_watch
				WHERE forum_watch.user_id = ?
				', 'forum_id', array($userId));
	}

	public function sendNotificationToWatchUsersOnNewThread(array $thread, /* array */ $forum, array $post)
	{
		// validates the thread
		if (empty($thread) OR $thread['discussion_state'] != 'visible') return;

		if (empty($forum)) $forum = $this->_getForumModel()->getForumById($thread['node_id']);
		if (empty($forum)) return;

		// validates the post
		if (empty($post))
		{
			$post = $this->_getPostModel()->getPostById(
					$thread['first_post_id'],
					array(
							'join' => XenForo_Model_Post::FETCH_USER,
					)
			);
		}
		if (empty($post) OR $post['message_state'] != 'visible') return;

		// honors the core option to include message in watched thread or not
		// since 1.0
		if (XenForo_Application::get('options')->emailWatchedThreadIncludeMessage)
		{
			$parseBbCode = true;
			$emailTemplate = 'forumwatch_watched_forum_new_thread_messagetext';
		}
		else
		{
			$parseBbCode = false;
			$emailTemplate = 'forumwatch_watched_forum_new_thread';
		}

		// fetches a full user record if we don't have one already
		if (!isset($post['avatar_width']) OR !isset($post['custom_title']))
		{
			$postUser = $this->_getUserModel()->ForumWatch_getUserById($post['user_id']);
			if ($postUser)
			{
				$post = array_merge($postUser, $post);
			}
			else
			{
				$post['avatar_width'] = 0;
				$post['custom_title'] = '';
			}
		}

		$users = $this->_getUsersWatchingForum($thread['node_id']);

		foreach ($users AS $user)
		{
			if ($user['user_id'] == $thread['user_id']) continue; // same user, do nothing
			if ($this->_getUserModel()->isUserIgnored($user, $post['user_id'])) continue; // the subcriber ignores the reply poster, do nothing

			$permissions = XenForo_Permission::unserializePermissions($user['node_permission_cache']);
			if (!$this->_getForumModel()->canViewForum($thread, $null, $permissions, $user)) continue; // failed permission check, do nothing
			if (!$this->_getThreadModel()->canViewThread($thread, $forum, $null, $permissions, $user)) continue; // failed permission check, do nothing

			if ($this->_viaEmail($user) AND $user['email'] AND $user['user_state'] == 'valid')
			{
				if (!isset($post['messageText']) AND $parseBbCode)
				{
					$bbCodeParserText = new XenForo_BbCode_Parser(XenForo_BbCode_Formatter_Base::create('Text'));
					$post['messageText'] = new XenForo_BbCode_TextWrapper($post['message'], $bbCodeParserText);

					$bbCodeParserHtml = new XenForo_BbCode_Parser(XenForo_BbCode_Formatter_Base::create('HtmlEmail'));
					$post['messageHtml'] = new XenForo_BbCode_TextWrapper($post['message'], $bbCodeParserHtml);
				}

				if (!isset($thread['titleCensored']))
				{
					$thread['titleCensored'] = XenForo_Helper_String::censorString($thread['title']);
				}

				$mail = new XenForo_Mail($emailTemplate, array(
						'post' => $post,
						'thread' => $thread,
						'forum' => $forum,
						'receiver' => $user
				), $user['language_id']);
				$mail->enableAllLanguagePreCache();
				$mail->queue($user['email'], $user['username']);
			}

			if ($this->_viaAlert($user))
			{
				XenForo_Model_Alert::alert(
				$user['user_id'],
				$thread['user_id'],
				$thread['username'],
				'post',
				$thread['first_post_id'],
				'forumwatch_thread');
			}
		}
	}

	public function sendNotificationToWatchUsersOnReply(array $reply, array $thread, array $ignoredUsers = array(), $noAlerts = array())
	{
		// validates the reply
		if (empty($reply) OR $reply['message_state'] != 'visible') return;

		// validates the thread
		if (empty($thread))
		{
			$thread = $threadModel->getThreadById($reply['thread_id'], array(
					'join' => XenForo_Model_Thread::FETCH_FORUM
			));
		}
		if (empty($thread) OR $thread['discussion_state'] != 'visible') return;

		// gets latest posts of the thread to determine whether to send notification or not
		// this method is called in XenForo_Model_ThreadWatch too
		// but we cached the result anyway so performance should not be hit
		$autoReadDate = XenForo_Application::$time - (XenForo_Application::get('options')->readMarkingDataLifetime * 86400);
		$latestPosts = $this->_getPostModel()->ForumWatch_getNewestPostsInThreadAfterDate(
				$thread['thread_id'], $autoReadDate
		);
		if (empty($latestPosts)) return;
		// ignore the newest posts, we don't need to check it
		list($key) = each($latestPosts);
		unset($latestPosts[$key]);
		$defaultPreviousPost = reset($latestPosts);

		// honors the core option to include message in watched thread or not
		// since 1.0
		if (XenForo_Application::get('options')->emailWatchedThreadIncludeMessage)
		{
			$parseBbCode = true;
			$emailTemplate = 'forumwatch_watched_forum_reply_messagetext';
		}
		else
		{
			$parseBbCode = false;
			$emailTemplate = 'forumwatch_watched_forum_reply';
		}

		// fetches a full user record if we don't have one already
		if (!isset($reply['avatar_width']) OR !isset($reply['custom_title']))
		{
			// this methos is also called in XenForo_Model_ThreadWatch
			// we cached the result anyway so no performance hit here...
			$replyUser = $this->_getUserModel()->ForumWatch_getUserById($reply['user_id']);
			if ($replyUser)
			{
				$reply = array_merge($replyUser, $reply);
			}
			else
			{
				$reply['avatar_width'] = 0;
				$reply['custom_title'] = '';
			}
		}

		$users = $this->_getUsersWatchingForum($thread['node_id'], $thread['thread_id']);

		foreach ($users AS $user)
		{
			if ($user['user_id'] == $reply['user_id']) continue; // same user, do nothing
			if ($this->_getUserModel()->isUserIgnored($user, $reply['user_id'])) continue; // the subcriber ignores the reply poster, do nothing
			if (in_array($user['user_id'], $ignoredUsers)) continue; // this user watched this thread too, so ignore him/her
			if (!$this->_wantReply($user)) continue; // this user doesn't subcribe for replies, do nothing

			// checks and tries to bypass sending notification...
			// this piece of code is copied from XenForo_Model_ForumWatch::sendNotificationToWatchUsersOnReply
			if (empty($defaultPreviousPost) OR !$this->_getUserModel()->isUserIgnored($user, $defaultPreviousPost['user_id']))
			{
				$previousPost = $defaultPreviousPost;
			}
			else
			{
				$previousPost = false;
				foreach ($latestPosts AS $latestPost)
				{
					if (!$userModel->isUserIgnored($user, $latestPost['user_id']))
					{
						$previousPost = $latestPost;
						break;
					}
				}
			}
			if (empty($previousPost) OR $previousPost['post_date'] < $autoReadDate)
			{
				// well, we have to send notification (*sad*)
			}
			elseif ($previousPost['post_date'] > $user['thread_read_date'])
			{
				// user hasn't read it yet, bail out!
				// added check for option, admin may want to send out notification for all replies
				// since 1.1
				if (ForumWatch_Option::get('alwaysNotifyNewReply'))
				{
					// yep, just send notification anyway
				}
				else
				{
					// skip this one
					continue;
				}
			}

			$permissions = XenForo_Permission::unserializePermissions($user['node_permission_cache']);
			if (!$this->_getThreadModel()->canViewThreadAndContainer($thread, $thread, $null, $permissions, $user)) continue; // failed permission check, do nothing

			if ($this->_viaEmail($user) AND $user['email'] AND $user['user_state'] == 'valid')
			{
				if (!isset($reply['messageText']) AND $parseBbCode)
				{
					$bbCodeParserText = new XenForo_BbCode_Parser(XenForo_BbCode_Formatter_Base::create('Text'));
					$reply['messageText'] = new XenForo_BbCode_TextWrapper($reply['message'], $bbCodeParserText);

					$bbCodeParserHtml = new XenForo_BbCode_Parser(XenForo_BbCode_Formatter_Base::create('HtmlEmail'));
					$reply['messageHtml'] = new XenForo_BbCode_TextWrapper($reply['message'], $bbCodeParserHtml);
				}

				if (!isset($thread['titleCensored']))
				{
					$thread['titleCensored'] = XenForo_Helper_String::censorString($thread['title']);
				}

				$mail = XenForo_Mail::create($emailTemplate, array(
						'reply' => $reply,
						'thread' => $thread,
						'forum' => $thread,
						'receiver' => $user
				), $user['language_id']);
				$mail->enableAllLanguagePreCache();
				$mail->queue($user['email'], $user['username']);
			}

			if ($this->_viaAlert($user) AND !in_array($user['user_id'], $noAlerts))
			{
				XenForo_Model_Alert::alert(
				$user['user_id'],
				$reply['user_id'],
				$reply['username'],
				'post',
				$reply['post_id'],
				'forumwatch_reply');
			}
		}
	}

	/**
	 * @return ForumWatch_Model_ForumWatchGroup
	 */
	protected function _getForumWatchGroupModel()
	{
		return $this->getModelFromCache('ForumWatch_Model_ForumWatchGroup');
	}

	/**
	 * @return XenForo_Model_Forum
	 */
	protected function _getForumModel()
	{
		return $this->getModelFromCache('XenForo_Model_Forum');
	}

	/**
	 * @return XenForo_Model_Thread
	 */
	protected function _getThreadModel()
	{
		return $this->getModelFromCache('XenForo_Model_Thread');
	}

	/**
	 * @return XenForo_Model_Post
	 */
	protected function _getPostModel()
	{
		return $this->getModelFromCache('XenForo_Model_Post');
	}

	/**
	 * @return XenForo_Model_User
	 */
	protected function _getUserModel()
	{
		return $this->getModelFromCache('XenForo_Model_User');
	}

	protected function _getUsersWatchingForum($forumId, $threadId = 0)
	{
		static $caches = array();

		$autoReadDate = XenForo_Application::$time - (XenForo_Application::get('options')->readMarkingDataLifetime * 86400);

		$hash = "$forumId|$threadId";
		if (!isset($caches[$hash]))
		{
			$watches = $this->_getDb()->fetchAll('
				SELECT forum_watch.*
				FROM xf_forum_watch AS forum_watch
				WHERE forum_watch.forum_id = ?
				', $forumId);
			$users = array();
			foreach ($watches as $watch)
			{
				$users[intval($watch['user_id'])] = $watch;
			}

			// since 2.0
			// get user ids in groups watching requested forum
			$groupUsers = $this->_getForumWatchGroupModel()->getUsersInWatchingGroupsOfForum($forumId);
			foreach ($groupUsers as $groupUserId => $groupUserConfig)
			{
				if (isset($users[$groupUserId]))
				{
					$users[$groupUserId]['reply_subscribe'] = max($users[$groupUserId]['reply_subscribe'], $groupUserConfig['reply_subscribe']);
					$users[$groupUserId]['notification_method'] = max($users[$groupUserId]['notification_method'], $groupUserConfig['notification_method']);
				}
				else
				{
					$users[$groupUserId] = $groupUserConfig;
				}
			}

			if (!empty($users))
			{
				$caches[$hash] = $this->fetchAllKeyed('
						SELECT user.*,
						user_option.*,
						user_profile.*,
						permission.cache_value AS node_permission_cache,
						GREATEST(COALESCE(thread_read.thread_read_date, 0), COALESCE(forum_read.forum_read_date, 0), ' . $autoReadDate . ') AS thread_read_date
						FROM xf_user AS user
						INNER JOIN xf_user_option AS user_option ON
						(user_option.user_id = user.user_id)
						INNER JOIN xf_user_profile AS user_profile ON
						(user_profile.user_id = user.user_id)
						LEFT JOIN xf_permission_cache_content AS permission
						ON (permission.permission_combination_id = user.permission_combination_id
						AND permission.content_type = \'node\'
						AND permission.content_id = ?)
						LEFT JOIN xf_thread_read AS thread_read
						ON (thread_read.thread_id = ? AND thread_read.user_id = user.user_id)
						LEFT JOIN xf_forum_read AS forum_read
						ON (forum_read.node_id = permission.content_id AND forum_read.user_id = user.user_id)
						WHERE user.user_id IN (' . $this->_getDb()->quote(array_keys($users)) . ')
							AND user.user_state = \'valid\'
							AND user.is_banned = 0
						', 'user_id', array($forumId, $threadId));

				foreach (array_keys($caches[$hash]) as $cacheUserId)
				{
					$cacheUserId = intval($cacheUserId);
					if (!isset($users[$cacheUserId]))
					{
						unset($caches[$hash][$cacheUserId]);
					}

					$caches[$hash][$cacheUserId] = array_merge($caches[$hash][$cacheUserId], $users[$cacheUserId]);
				}
			}
			else
			{
				$caches[$hash] = array();
			}
		}

		return $caches[$hash];
	}

	protected function _viaEmail($forumWatch)
	{
		switch ($forumWatch['notification_method'])
		{
			case 0: // email only
			case 2: // email and alert
				return true;
			default:
				return false;
		}
	}

	protected function _viaAlert($forumWatch)
	{
		switch ($forumWatch['notification_method'])
		{
			case 1: // alert only
			case 2: // email and alert
				return true;
			default:
				return false;
		}
	}

	protected function _wantReply($forumWatch)
	{
		return (bool)$forumWatch['reply_subscribe'];
	}
}