<?php

class ForumWatch_Model_ForumWatchGroup extends XenForo_Model
{
	const FETCH_USER_GROUP = 0x01;
	const FETCH_FORUM = 0x02;

	const CACHE_KEY_FORUM_GROUP_MAPPING = 'ForumWatch_fgm';
	const CACHE_KEY_GROUP_USER_MAPPING = 'ForumWatch_gum';

	public function rebuildCacheForumGroup()
	{
		$watches = $this->getForumWatchesGroup();

		$cache = array();
		$userGroupIds = array();
		foreach ($watches as $watch)
		{
			$watch['forum_id'] = intval($watch['forum_id']);
			$watch['user_group_id'] = intval($watch['user_group_id']);
			$cache[$watch['forum_id']][$watch['user_group_id']] = $watch;
			$userGroupIds[] = $watch['user_group_id'];
		}
		XenForo_Application::setSimpleCacheData(self::CACHE_KEY_FORUM_GROUP_MAPPING, $cache);

		// rebuild group user cache if needed
		$groupUser = $this->_getDataRegistryModel()->get(self::CACHE_KEY_GROUP_USER_MAPPING);
		if (empty($groupUser)) $groupUser = array();
		$groupUserChanged = false;
		$missingUserGroupIds = array();
		foreach ($userGroupIds as $userGroupId)
		{
			if (!isset($groupUser[$userGroupId]))
			{
				$missingUserGroupIds[] = $userGroupId;
			}
		}
		foreach (array_keys($groupUser) as $groupUserKey)
		{
			if (!in_array($groupUserKey, $userGroupIds))
			{
				// remove cache entry for unused user groups
				unset($groupUser[$groupUserKey]);
				$groupUserChanged = true;
			}
		}
		if ($groupUserChanged)
		{
			$this->_getDataRegistryModel()->set(self::CACHE_KEY_GROUP_USER_MAPPING, $groupUser);
		}
		unset($groupUser); // free memory
		if (!empty($missingUserGroupIds))
		{
			$this->rebuildCacheGroupUser($missingUserGroupIds);
		}

		return $cache;
	}

	public function rebuildCacheGroupUser(array $changeUserGroupIds)
	{
		$cache = $this->_getDataRegistryModel()->get(self::CACHE_KEY_GROUP_USER_MAPPING);
		if (empty($cache)) $cache = array();

		$changeUserGroupIds = array_unique($changeUserGroupIds);
		foreach ($changeUserGroupIds as $userGroupId)
		{
			$userGroupId = intval($userGroupId);
			$cache[$userGroupId] = array_map('intval', $this->_getDb()->fetchCol('
					SELECT user_id
					FROM xf_user AS user
					WHERE user.user_group_id = ?
					OR FIND_IN_SET(' . $this->_getDb()->quote($userGroupId) . ', user.secondary_group_ids)
					', $userGroupId));
		}

		$this->_getDataRegistryModel()->set(self::CACHE_KEY_GROUP_USER_MAPPING, $cache);

		return $cache;
	}

	public function rebuildCacheGroupUserOneUser($userId, array $addedGroupIds, array $removedGroupIds)
	{
		$cache = $this->_getDataRegistryModel()->get(self::CACHE_KEY_GROUP_USER_MAPPING);
		if (empty($cache)) $cache = array();
		$cacheChanged = false;

		foreach ($removedGroupIds as $groupId)
		{
			if (!isset($cache[$groupId])) continue;

			$key = array_search($userId, $cache[$groupId]);
			if ($key !== false)
			{
				unset($cache[$groupId][$key]);
				$cacheChanged = true;
			}
		}

		foreach ($addedGroupIds as $groupId)
		{
			if (!isset($cache[$groupId])) continue;

			$key = array_search($userId, $cache[$groupId]);
			if ($key === false)
			{
				$cache[$groupId][] = $userId;
				$cacheChanged = true;
			}
		}

		if ($cacheChanged)
		{
			$this->_getDataRegistryModel()->set(self::CACHE_KEY_GROUP_USER_MAPPING, $cache);
		}

		return $cache;
	}

	public function isGroupsWatchingForum($userGroupId, $secondaryGroupIds, $forumId)
	{
		$userGroupIds = array_map('intval', explode(',', $secondaryGroupIds));
		$userGroupIds[] = intval($userGroupId);
		$forumId = intval($forumId);

		$forumGroup = XenForo_Application::getSimpleCacheData(self::CACHE_KEY_FORUM_GROUP_MAPPING);
		if ($forumGroup === false) $forumGroup = $this->rebuildCacheForumGroup();

		if (isset($forumGroup[$forumId]))
		{
			foreach ($userGroupIds as $userGroupId)
			{
				if (isset($forumGroup[$forumId][$userGroupId]))
				{
					return true;
				}
			}
		}

		return false;
	}

	public function getUserInWatchingGroupsOfForum($userId, $forumId)
	{
		$userId = intval($userId);
		$users = $this->getUsersInWatchingGroupsOfForum($forumId);

		if (isset($users[$userId]))
		{
			return $users[$userId];
		}
		else
		{
			return false;
		}
	}

	public function getUsersInWatchingGroupsOfForum($forumId)
	{
		$forumId = intval($forumId);

		$users = array();
		$forumGroup = XenForo_Application::getSimpleCacheData(self::CACHE_KEY_FORUM_GROUP_MAPPING);
		if ($forumGroup === false) $forumGroup = $this->rebuildCacheForumGroup();

		if (isset($forumGroup[$forumId]))
		{
			$userGroupIds = array_keys($forumGroup[$forumId]);
			$missingUserGroupIds = array();

			$groupUser = $this->_getDataRegistryModel()->get(self::CACHE_KEY_GROUP_USER_MAPPING);
			foreach ($userGroupIds as $userGroupId)
			{
				if (!isset($groupUser[$userGroupId]))
				{
					$missingUserGroupIds[] = $userGroupId;
				}
			}
			if (!empty($missingUserGroupIds))
			{
				$groupUser = $this->rebuildCacheGroupUser($missingUserGroupIds);
			}

			foreach ($userGroupIds as $userGroupId)
			{
				if (isset($groupUser[$userGroupId]))
				{
					foreach ($groupUser[$userGroupId] as $userId)
					{
						if (isset($users[$userId]))
						{
							// existing entry found
							foreach ($forumGroup[$forumId][$userGroupId] as $key => $value)
							{
								$users[$userId][$key] = max($users[$userId][$key], $value);
							}
						}
						else
						{
							// new
							$users[$userId] = $forumGroup[$forumId][$userGroupId];
						}
					}
				}
			}
		}

		return $users;
	}

	public function getForumWatchGroupByIds($userGroupId, $forumId, array $fetchOptions = array())
	{
		$data = $this->getForumWatchesGroup(array(
				'user_group_id' => $userGroupId,
				'forum_id' => $forumId
		), $fetchOptions);

		return reset($data);
	}

	public function getForumWatchesGroup(array $conditions = array(), array $fetchOptions = array())
	{
		$whereConditions = $this->prepareForumWatchGroupConditions($conditions, $fetchOptions);

		$orderClause = $this->prepareForumWatchGroupOrderOptions($fetchOptions);
		$joinOptions = $this->prepareForumWatchGroupFetchOptions($fetchOptions);
		$limitOptions = $this->prepareLimitFetchOptions($fetchOptions);

		$all = $this->_getDb()->fetchAll($this->limitQueryResults("
				SELECT xf_forum_watch_group.*
				$joinOptions[selectFields]
				FROM `xf_forum_watch_group` AS xf_forum_watch_group
				$joinOptions[joinTables]
				WHERE $whereConditions
				$orderClause
				", $limitOptions['limit'], $limitOptions['offset']
		));

		return $all;
	}

	public function countForumWatchesGroup(array $conditions = array(), array $fetchOptions = array())
	{
		$whereConditions = $this->prepareForumWatchGroupConditions($conditions, $fetchOptions);

		$orderClause = $this->prepareForumWatchGroupOrderOptions($fetchOptions);
		$joinOptions = $this->prepareForumWatchGroupFetchOptions($fetchOptions);
		$limitOptions = $this->prepareLimitFetchOptions($fetchOptions);

		return $this->_getDb()->fetchOne("
				SELECT COUNT(*)
				FROM `xf_forum_watch_group` AS xf_forum_watch_group
				$joinOptions[joinTables]
				WHERE $whereConditions
				");
	}

	public function prepareForumWatchGroupConditions(array $conditions = array(), array $fetchOptions = array())
	{
		$sqlConditions = array();
		$db = $this->_getDb();

		if (isset($conditions['user_group_id']))
		{
			if (is_array($conditions['user_group_id']))
			{
				if (!empty($conditions['user_group_id']))
				{
					// only use IN condition if the array is not empty (nasty!)
					$sqlConditions[] = "xf_forum_watch_group.user_group_id IN (" . $db->quote($conditions['user_group_id']) . ")";
				}
			}
			else
			{
				$sqlConditions[] = "xf_forum_watch_group.user_group_id = " . $db->quote($conditions['user_group_id']);
			}
		}

		if (isset($conditions['forum_id']))
		{
			if (is_array($conditions['forum_id']))
			{
				if (!empty($conditions['forum_id']))
				{
					// only use IN condition if the array is not empty (nasty!)
					$sqlConditions[] = "xf_forum_watch_group.forum_id IN (" . $db->quote($conditions['forum_id']) . ")";
				}
			}
			else
			{
				$sqlConditions[] = "xf_forum_watch_group.forum_id = " . $db->quote($conditions['forum_id']);
			}
		}

		if (isset($conditions['reply_subscribe']))
		{
			if (is_array($conditions['reply_subscribe']))
			{
				if (!empty($conditions['reply_subscribe']))
				{
					// only use IN condition if the array is not empty (nasty!)
					$sqlConditions[] = "xf_forum_watch_group.reply_subscribe IN (" . $db->quote($conditions['reply_subscribe']) . ")";
				}
			}
			else
			{
				$sqlConditions[] = "xf_forum_watch_group.reply_subscribe = " . $db->quote($conditions['reply_subscribe']);
			}
		}

		if (isset($conditions['notification_method']))
		{
			if (is_array($conditions['notification_method']))
			{
				if (!empty($conditions['notification_method']))
				{
					// only use IN condition if the array is not empty (nasty!)
					$sqlConditions[] = "xf_forum_watch_group.notification_method IN (" . $db->quote($conditions['notification_method']) . ")";
				}
			}
			else
			{
				$sqlConditions[] = "xf_forum_watch_group.notification_method = " . $db->quote($conditions['notification_method']);
			}
		}

		return $this->getConditionsForClause($sqlConditions);
	}

	public function prepareForumWatchGroupFetchOptions(array $fetchOptions = array())
	{
		$selectFields = '';
		$joinTables = '';

		if (!empty($fetchOptions['join']))
		{
			if ($fetchOptions['join'] & self::FETCH_USER_GROUP)
			{
				$selectFields .= ',
						user_group.*, user_group.title AS user_group_title';
				$joinTables .= '
						INNER JOIN `xf_user_group` AS user_group
						ON (user_group.user_group_id = xf_forum_watch_group.user_group_id)';
			}

			if ($fetchOptions['join'] & self::FETCH_FORUM)
			{
				$selectFields .= ',
						forum.*, node.title AS forum_title';
				$joinTables .= '
						INNER JOIN `xf_node` AS node
						ON (node.node_id = xf_forum_watch_group.forum_id)
						INNER JOIN `xf_forum` AS forum
						ON (forum.node_id = xf_forum_watch_group.forum_id)';
			}
		}

		return array(
				'selectFields' => $selectFields,
				'joinTables'   => $joinTables
		);
	}

	public function prepareForumWatchGroupOrderOptions(array $fetchOptions = array(), $defaultOrderSql = '')
	{
		$choices = array(
				'natural' => 'user_group.title %s, node.lft',
		);

		return $this->getOrderByClause($choices, $fetchOptions, $defaultOrderSql);
	}

}